SystemOrganization addCategory: #QuantumLogic!
SystemOrganization addCategory: #'QuantumLogic-Engine'!
SystemOrganization addCategory: #'QuantumLogic-Example'!
SystemOrganization addCategory: #'QuantumLogic-Tests'!

TestCase subclass: #QuantumMapColoringTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QuantumLogic-Tests'!

!QuantumMapColoringTest methodsFor: 'as yet unclassified' stamp: 'hjo 2/15/2012 23:09'!
testJust

|colors seed engine|
colors := #(red green blue).

seed := QuantumMapColoring new.
seed alabama: (QuantumObject possibilities: colors).
seed florida: (QuantumObject possibilities: colors).
seed georgia: (QuantumObject possibilities: colors).
seed mississippi: (QuantumObject possibilities: colors).
seed tennessee: (QuantumObject possibilities: colors).

seed possibilities.


engine := DepthFirstQuantumEngine new.
engine stack push: seed.
engine run.

self assert: 6 equals: engine answers size.! !

TestCase subclass: #QuantumNonogramTest
	instanceVariableNames: 'nono'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QuantumLogic-Tests'!

!QuantumNonogramTest methodsFor: 'as yet unclassified' stamp: 'hjo 2/1/2012 18:26'!
setUp 

	super setUp.
	nono := QuantumNonogram rows: 6 columns: 6.
	nono createRowAt: 1 withHint: {2. 1}.
	nono createRowAt: 2 withHint: {1. 3}.
	nono createRowAt: 3 withHint: {1. 2}.
	nono createRowAt: 4 withHint: {3}.
	nono createRowAt: 5 withHint: {4}.
	nono createRowAt: 6 withHint: {1}.

	nono createColumnAt: 1 withHint: {1}.
	nono createColumnAt: 2 withHint: {5}.
	nono createColumnAt: 3 withHint: {2}.
	nono createColumnAt: 4 withHint: {5}.
	nono createColumnAt: 5 withHint: {2. 1}.
	nono createColumnAt: 6 withHint: {2}.! !

!QuantumNonogramTest methodsFor: 'as yet unclassified' stamp: 'hjo 2/1/2012 21:35'!
testArray

	| seed engine ans |
	seed := QuantumNonogramArray cells: {QuantumObject possibilities:{true. false}. QuantumObject possibilities:{true. false}} hints: {2}.

	engine := DepthFirstQuantumEngine new.
	engine stack push: seed.
	engine run.
	ans := engine answers.
	
	self assert: 1 equals: ans size.
	self assert: true equals: (ans anyOne cells at: 1) determinedValue.
	self assert: true equals: (ans anyOne cells at: 2) determinedValue.! !

!QuantumNonogramTest methodsFor: 'as yet unclassified' stamp: 'hjo 2/1/2012 20:42'!
testMain

	nono runSolving.
	self assert: 
'a QuantumNonogram
1 1 0 0 0 1 
0 1 0 1 1 1 
0 1 0 1 1 0 
0 1 1 1 0 0 
0 1 1 1 1 0 
0 0 0 1 0 0 '
	 equals:  nono printString.! !

TestCase subclass: #QuantumObjectTest
	instanceVariableNames: 'only three'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QuantumLogic-Tests'!

!QuantumObjectTest methodsFor: 'running' stamp: 'hjo 2/1/2012 14:01'!
setUp

	super setUp.
	three := QuantumObject possibilities: {1. 2. 3}.
	only := QuantumObject possibilities: {4}.! !

!QuantumObjectTest methodsFor: 'tests' stamp: 'hjo 2/1/2012 14:01'!
testBe

	self assert: 3 equals: three possibilities size.
	three be: 2.
	self assert: 1 equals: three possibilities size.
	self flag: 'error when object is not in possibility'.! !

!QuantumObjectTest methodsFor: 'tests' stamp: 'hjo 2/1/2012 14:05'!
testBeNotInThePossibilities

	self should: [three be: 200] raise: AssertionFailure.! !

!QuantumObjectTest methodsFor: 'as yet unclassified' stamp: 'hjo 2/1/2012 18:23'!
testDeterminedValue

	self assert: 4 equals: only determinedValue.
	self should:  [three determinedValue ] raise: AssertionFailure.
	  ! !

!QuantumObjectTest methodsFor: 'tests' stamp: 'hjo 2/1/2012 14:01'!
testIsDetermined

	self deny: three isDetermined.
	self assert: only isDetermined.! !

!QuantumObjectTest methodsFor: 'tests' stamp: 'hjo 2/1/2012 13:54'!
testPossibilities

	self should: [QuantumObject possibilities: {}  ] raise: AssertionFailure ! !

!QuantumObjectTest methodsFor: 'tests' stamp: 'hjo 2/1/2012 14:01'!
testRemovePossibility

	three removePossibility: 3.  
	self assert: 2 equals: three possibilities size.
	self should:  [only removePossibility: 4] raise: OutOfQuantumPossibility.
	  ! !

TestCase subclass: #QuantumSudokuTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QuantumLogic-Tests'!

!QuantumSudokuTest methodsFor: 'as yet unclassified' stamp: 'hjo 2/15/2012 23:15'!
testOne
|sudoku dfa|
sudoku := 
QuantumSudoku matrix: ((Matrix rows: 9 columns: 9 contents:	
		#(
			1  8  6  nil  nil  nil    9  2  nil
			nil    5  nil    9  nil    6  nil  nil  8
			4  nil  nil  nil  nil  nil  nil  nil  nil
			8  nil  nil  2  nil  nil    4    5  7
			nil  nil    7  nil    4  nil  nil  nil    1
			nil  nil    4  nil  nil    8  nil  nil    3
			2  nil    8    1  nil    9  nil    7    4
			nil  nil  nil  nil  nil  nil  nil    1  nil
			nil    9    1    5    2    4    8  nil  nil
			))
		collect: [:v  |
			QuantumObject possibilities:  (v isNil ifTrue: [ (1 to: 9) ] ifFalse: [{v} ]).
			]).

dfa := DepthFirstQuantumEngine new.
dfa stack push: sudoku.
dfa run.
dfa answers. 
! !

Error subclass: #OutOfQuantumPossibility
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QuantumLogic'!

Object subclass: #BaseBallJudge
	instanceVariableNames: 'answer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QuantumLogic-Example'!

!BaseBallJudge classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/16/2012 08:31'!
answer: anArray 

	^self new answer: anArray! !

!BaseBallJudge methodsFor: 'as yet unclassified' stamp: 'hjo 2/16/2012 08:31'!
answer: anArray 

	answer := anArray! !

!BaseBallJudge methodsFor: 'as yet unclassified' stamp: 'hjo 2/22/2012 17:06'!
judge: anArray 

	|ment|
	ment := BaseBallJudgement new.
	ment submission: anArray.
	ment strikes: 0.
	ment balls: 0.
	
	1 to: 4 do: [:k|
		1 to: 4 do: [:t|
				(answer at: k) = (anArray at: t) ifTrue: 
					[ k = t 
					ifTrue: [ ment increaseStrikes ] 
					ifFalse: [ ment increaseBalls ].
					]	
			]
		].
	^ment! !

Object subclass: #BaseBallJudgement
	uses: HashProofTrait
	instanceVariableNames: 'strikes balls submission'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QuantumLogic-Example'!

BaseBallJudgement class
	uses: HashProofTrait classTrait
	instanceVariableNames: ''!

!BaseBallJudgement classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/22/2012 17:02'!
submission: anArray strikes: int1 balls: int2

	^self new
		submission: anArray;
		strikes: int1;
		balls: int2	! !

!BaseBallJudgement methodsFor: 'as yet unclassified' stamp: 'hjo 2/22/2012 16:55'!
balls

	^balls! !

!BaseBallJudgement methodsFor: 'as yet unclassified' stamp: 'hjo 2/22/2012 16:47'!
balls: aSmallInteger 

	balls := aSmallInteger! !

!BaseBallJudgement methodsFor: 'as yet unclassified' stamp: 'hjo 2/22/2012 17:04'!
gettersForEquality

	^#(submission strikes balls)! !

!BaseBallJudgement methodsFor: 'as yet unclassified' stamp: 'hjo 2/22/2012 17:06'!
increaseBalls

	balls := balls + 1! !

!BaseBallJudgement methodsFor: 'as yet unclassified' stamp: 'hjo 2/22/2012 17:07'!
increaseStrikes

	strikes := strikes + 1! !

!BaseBallJudgement methodsFor: 'as yet unclassified' stamp: 'hjo 2/22/2012 17:05'!
initialize 

	super initialize.
	strikes := 0.
	balls := 0.! !

!BaseBallJudgement methodsFor: 'as yet unclassified' stamp: 'hjo 2/22/2012 16:55'!
strikes

	^strikes! !

!BaseBallJudgement methodsFor: 'as yet unclassified' stamp: 'hjo 2/22/2012 16:47'!
strikes: aSmallInteger 

	strikes := aSmallInteger! !

!BaseBallJudgement methodsFor: 'as yet unclassified' stamp: 'hjo 2/22/2012 17:03'!
submission

	^submission! !

!BaseBallJudgement methodsFor: 'as yet unclassified' stamp: 'hjo 2/22/2012 17:03'!
submission: anArray

	submission := anArray! !

Object subclass: #DepthFirstQuantumEngine
	instanceVariableNames: 'answers stack'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QuantumLogic-Engine'!

!DepthFirstQuantumEngine methodsFor: 'as yet unclassified' stamp: 'hjo 3/27/2008 11:02'!
addToAnswers: anObject

	self answers add: anObject! !

!DepthFirstQuantumEngine methodsFor: 'accessing' stamp: 'hjo 2/2/2008 23:13'!
answers

	answers isNil ifTrue: [answers := OrderedCollection new].
	^answers! !

!DepthFirstQuantumEngine methodsFor: 'looping' stamp: 'hjo 2/2/2008 23:15'!
run

	[self stack isEmpty]
		whileFalse: [ self step ]! !

!DepthFirstQuantumEngine methodsFor: 'looping' stamp: 'hjo 2/2/2008 21:48'!
runToFindOneAnswer

	[ self answers isEmpty ] whileTrue: [ self step ]! !

!DepthFirstQuantumEngine methodsFor: 'accessing' stamp: 'hjo 2/2/2008 23:12'!
stack

	stack isNil ifTrue: [stack := Stack new].
	^stack! !

!DepthFirstQuantumEngine methodsFor: 'accessing' stamp: 'hjo 1/28/2012 19:38'!
stackContents

	^String streamContents: [:stream| 
		self stack do: [:sl| sl printOn: stream. stream cr ]
	]! !

!DepthFirstQuantumEngine methodsFor: 'testing' stamp: 'hjo 2/2/2008 23:15'!
stackIsEmpty

	^self stack isEmpty! !

!DepthFirstQuantumEngine methodsFor: 'as yet unclassified' stamp: 'hjo 1/29/2012 17:47'!
stackPopPossibilities

	^self stack pop possibilities! !

!DepthFirstQuantumEngine methodsFor: 'accessing' stamp: 'hjo 2/2/2008 23:14'!
stackSize
	
	^self stack size! !

!DepthFirstQuantumEngine methodsFor: 'accessing' stamp: 'hjo 1/29/2012 17:50'!
stackTop

	^self stack first! !

!DepthFirstQuantumEngine methodsFor: 'looping' stamp: 'hjo 1/29/2012 17:47'!
step

	"override this for more control over processing agents"
	
	self stackPopPossibilities do: 
		[:agent| 
			agent isDetermined
				ifTrue: [self addToAnswers: agent]
				ifFalse: [self stack push: agent]
		]! !

DepthFirstQuantumEngine subclass: #MaxFindingDepthFirstQuantumEngine
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QuantumLogic-Engine'!

!MaxFindingDepthFirstQuantumEngine methodsFor: 'as yet unclassified' stamp: 'hjo 3/27/2008 10:48'!
addToAnswers: anEcologicalBinPacking 

	self answers put: anEcologicalBinPacking! !

!MaxFindingDepthFirstQuantumEngine methodsFor: 'as yet unclassified' stamp: 'hjo 3/27/2008 10:17'!
answers

	answers isNil ifTrue: [ answers := MaxClub new ].
	^answers! !

!MaxFindingDepthFirstQuantumEngine methodsFor: 'as yet unclassified' stamp: 'hjo 3/27/2008 10:17'!
evaluator: aBlockContext

	self answers evaluator: aBlockContext! !

!MaxFindingDepthFirstQuantumEngine methodsFor: 'as yet unclassified' stamp: 'hjo 1/29/2012 17:47'!
stackPopPossibilities

	| spreaded |
	spreaded := super stackPopPossibilities.
	^self answers maximum isNil
		ifTrue: [spreaded]
		ifFalse: [spreaded select: [:each| self answers mayAccept: each ]]! !

Object subclass: #QuantumBaseball
	instanceVariableNames: 'digits judge judgements'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QuantumLogic-Example'!

!QuantumBaseball methodsFor: 'be' stamp: 'hjo 2/22/2012 17:39'!
checkJudgements

	self isDetermined ifTrue: [ | assumedJudge |
		assumedJudge := BaseBallJudge answer: self determinedValue.
		(judgements allSatisfy: [:each| 
			assumedJudge judge: each
			 ]) 
			ifTrue: [ judgements add: (judge judge: self determinedValue) ]
			ifFalse: [ OutOfQuantumPossibility new signal ].
		]! !

!QuantumBaseball methodsFor: 'as yet unclassified' stamp: 'hjo 2/25/2012 09:20'!
determinedValue

	self assert: [self isDetermined].
	^digits collect: #determinedValue! !

!QuantumBaseball methodsFor: 'be' stamp: 'hjo 2/25/2012 09:29'!
digitAt:anInteger be: aNumber 

	digits keysAndValuesDo: [:k :qo| k=anInteger ifTrue: [qo be: aNumber] ifFalse: [qo removePossibility: aNumber]].  
	self checkJudgements.! !

!QuantumBaseball methodsFor: 'as yet unclassified' stamp: 'hjo 2/25/2012 09:14'!
digits: anArray 

	digits := anArray! !

!QuantumBaseball methodsFor: 'as yet unclassified' stamp: 'hjo 2/25/2012 09:16'!
initialize 

	super initialize.
	judgements := OrderedCollection new.
	digits := 4 timesCollect: [QuantumObject possibilities:(1to:9)].! !

!QuantumBaseball methodsFor: 'as yet unclassified' stamp: 'hjo 2/25/2012 09:20'!
isDetermined

	^digits allSatisfy: [:each| each isDetermined ]  ! !

!QuantumBaseball methodsFor: 'as yet unclassified' stamp: 'hjo 2/16/2012 08:32'!
judge: aBaseBallJudge 

	judge := aBaseBallJudge! !

!QuantumBaseball methodsFor: 'as yet unclassified' stamp: 'hjo 2/25/2012 09:35'!
possibilities

	self isDetermined 
		ifTrue: [ ^Array with: self ]
		ifFalse: [
			1 to: 4 do: [:k|
				(digits at:k) isDetermined ifFalse: [ ^(digits at:k) collectPossibilities: [:num| self quantumCopy digitAt: k be: num ]  ].		
				].
		].! !

!QuantumBaseball methodsFor: 'as yet unclassified' stamp: 'hjo 2/25/2012 09:35'!
printOn: aStream

	aStream nextPutAll: '[['.  
	digits do: [:d| 
		aStream nextPutAll: (d isDetermined ifTrue: [d determinedValue printString] ifFalse: ['*']).
		].
	aStream nextPutAll: ']]'.  ! !

!QuantumBaseball methodsFor: 'as yet unclassified' stamp: 'hjo 2/25/2012 09:35'!
quantumCopy

	^self class new 
		digits: digits deepCopy;
		judge: judge! !

Object subclass: #QuantumMapColoring
	instanceVariableNames: 'alabama mississippi georgia tennessee florida'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QuantumLogic-Example'!

!QuantumMapColoring methodsFor: 'accessing' stamp: 'hjo 2/15/2012 22:19'!
alabama
	^ alabama! !

!QuantumMapColoring methodsFor: 'accessing' stamp: 'hjo 2/15/2012 22:19'!
alabama: anObject
	alabama := anObject! !

!QuantumMapColoring methodsFor: 'quantumlogic' stamp: 'hjo 2/15/2012 22:52'!
alabamaBe: aByteSymbol 

	alabama be: aByteSymbol.
	{mississippi . tennessee . georgia . florida} do: [:qo| qo removePossibility: aByteSymbol  ].
	  ! !

!QuantumMapColoring methodsFor: 'accessing' stamp: 'hjo 2/15/2012 22:19'!
florida
	^ florida! !

!QuantumMapColoring methodsFor: 'accessing' stamp: 'hjo 2/15/2012 22:19'!
florida: anObject
	florida := anObject! !

!QuantumMapColoring methodsFor: 'accessing' stamp: 'hjo 2/15/2012 22:19'!
georgia
	^ georgia! !

!QuantumMapColoring methodsFor: 'accessing' stamp: 'hjo 2/15/2012 22:19'!
georgia: anObject
	georgia := anObject! !

!QuantumMapColoring methodsFor: 'quantumlogic' stamp: 'hjo 2/15/2012 22:57'!
georgiaBe: aByteSymbol 

	georgia be: aByteSymbol.
	{alabama  . tennessee . florida} do: [:qo| qo removePossibility: aByteSymbol  ].
! !

!QuantumMapColoring methodsFor: 'quantumlogic' stamp: 'hjo 2/15/2012 22:43'!
isDetermined

	^{alabama . mississippi . georgia . tennessee . florida} allSatisfy: [:each| each isDetermined ]  ! !

!QuantumMapColoring methodsFor: 'accessing' stamp: 'hjo 2/15/2012 22:19'!
mississippi
	^ mississippi! !

!QuantumMapColoring methodsFor: 'accessing' stamp: 'hjo 2/15/2012 22:19'!
mississippi: anObject
	mississippi := anObject! !

!QuantumMapColoring methodsFor: 'quantumlogic' stamp: 'hjo 2/15/2012 22:54'!
mississippiBe: aByteSymbol 

	mississippi be: aByteSymbol.
	{tennessee . alabama} do: [:qo| qo removePossibility: aByteSymbol  ].
! !

!QuantumMapColoring methodsFor: 'quantumlogic' stamp: 'hjo 2/15/2012 23:04'!
possibilities

	alabama isDetermined ifFalse: [ ^alabama collectPossibilities: [:each|
			self deepCopy alabamaBe: each]].
	mississippi isDetermined ifFalse: [ ^mississippi collectPossibilities: [:each|
			self deepCopy mississippiBe: each]].
	georgia isDetermined ifFalse: [ ^georgia collectPossibilities: [:each|
			self deepCopy georgiaBe: each]].! !

!QuantumMapColoring methodsFor: 'accessing' stamp: 'hjo 2/15/2012 22:19'!
tennessee
	^ tennessee! !

!QuantumMapColoring methodsFor: 'accessing' stamp: 'hjo 2/15/2012 22:19'!
tennessee: anObject
	tennessee := anObject! !

Object subclass: #QuantumNonogram
	instanceVariableNames: 'matrix rows columns'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QuantumLogic-Example'!

!QuantumNonogram classSide methodsFor: 'as yet unclassified' stamp: 'hjo 1/29/2012 22:09'!
rows: rowCount columns: columnCount 

	^self new
		matrix: (Matrix rows: rowCount columns: columnCount contents: (rowCount * columnCount timesCollect: [QuantumObject possibilities: {true. false}])).! !

!QuantumNonogram methodsFor: 'accessing' stamp: 'hjo 1/29/2012 21:19'!
columns

	^ columns isNil ifTrue: [columns := Array new: matrix columnCount ] ifFalse: [columns]! !

!QuantumNonogram methodsFor: 'initialize-release' stamp: 'hjo 2/1/2012 21:35'!
createColumnAt: anInteger withHint: anArray 

	self columns at: anInteger put: (QuantumNonogramArray cells: (matrix atColumn: anInteger) hints: anArray)! !

!QuantumNonogram methodsFor: 'initialize-release' stamp: 'hjo 2/1/2012 21:35'!
createRowAt: anInteger withHint: anArray 

	self rows at: anInteger put: (QuantumNonogramArray cells: (matrix atRow:anInteger) hints: anArray)! !

!QuantumNonogram methodsFor: 'running' stamp: 'hjo 2/1/2012 20:09'!
fillInTheCellIfCan

	self rows do: #fillInTheCellIfCan.
	self columns do: #fillInTheCellIfCan.! !

!QuantumNonogram methodsFor: 'testing' stamp: 'hjo 1/29/2012 21:31'!
isDetermined

	^matrix allSatisfy: [:each| each isDetermined ]  ! !

!QuantumNonogram methodsFor: 'accessing' stamp: 'hjo 1/29/2012 22:08'!
matrix: aMatrix 

	matrix := aMatrix! !

!QuantumNonogram methodsFor: 'printing' stamp: 'hjo 1/31/2012 00:07'!
printOn: aStream

	super printOn: aStream.
	matrix isNil
	ifFalse: [
	1 to: matrix rowCount do: [:r|			aStream cr.
		1 to: matrix columnCount do: [:c| |q|
			q := matrix at: r at: c.
			q isDetermined
				ifTrue: [q possibilities anyOne ifTrue: [aStream nextPut: $1] ifFalse: [aStream nextPut: $0]]
				ifFalse: [aStream nextPut: $* ].
			aStream space.
			].
		]]
		
! !

!QuantumNonogram methodsFor: 'changing' stamp: 'hjo 2/1/2012 20:47'!
putBool: aBoolean at: rowIndex at: columnIndex

	(matrix at: rowIndex at: columnIndex) be: aBoolean.! !

!QuantumNonogram methodsFor: 'accessing' stamp: 'hjo 1/29/2012 21:05'!
rows

	^ rows isNil ifTrue: [rows := Array new: matrix rowCount ] ifFalse: [rows]! !

!QuantumNonogram methodsFor: 'running' stamp: 'hjo 2/1/2012 20:42'!
runSolving

	[self isDetermined] whileFalse: [self stepSolving]  ! !

!QuantumNonogram methodsFor: 'running' stamp: 'hjo 2/1/2012 20:41'!
stepSolving

	self fillInTheCellIfCan! !

Object subclass: #QuantumNonogramArray
	instanceVariableNames: 'cells hints'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QuantumLogic-Example'!

!QuantumNonogramArray classSide methodsFor: 'as yet unclassified' stamp: 'hjo 1/29/2012 21:05'!
cells: anArray hints: anArray2 

	^self new
		cells: anArray;
		hints: anArray2
		! !

!QuantumNonogramArray methodsFor: 'as yet unclassified' stamp: 'hjo 2/1/2012 19:19'!
cells

	^ cells! !

!QuantumNonogramArray methodsFor: 'as yet unclassified' stamp: 'hjo 1/29/2012 21:06'!
cells: anArray 

	cells := anArray! !

!QuantumNonogramArray methodsFor: 'as yet unclassified' stamp: 'hjo 2/1/2012 18:43'!
dupCopy

	^self species cells: cells deepCopy hints: hints! !

!QuantumNonogramArray methodsFor: 'as yet unclassified' stamp: 'hjo 2/1/2012 21:33'!
fillInTheCellIfCan
	
	|engine|
	self isDetermined ifTrue: [^self].
	engine := DepthFirstQuantumEngine new.
	engine stack push: self dupCopy.
	engine run.
			
	engine answers isEmpty ifTrue: [OutOfQuantumPossibility new signal].

	((1 to: cells size) collect: [:k|  QuantumObject fromQuantumObjects: engine answers gettingBlock: [:arr| arr cells at: k]  ]) 
		keysAndValuesDo: [:k :common | common isDetermined ifTrue: [self putBool: common determinedValue at: k] ].! !

!QuantumNonogramArray methodsFor: 'as yet unclassified' stamp: 'hjo 1/29/2012 22:20'!
hints

	^ hints! !

!QuantumNonogramArray methodsFor: 'as yet unclassified' stamp: 'hjo 1/29/2012 21:06'!
hints: anArray 

	hints := anArray! !

!QuantumNonogramArray methodsFor: 'as yet unclassified' stamp: 'hjo 2/1/2012 18:52'!
isDetermined

	^cells allSatisfy: [:qc| qc isDetermined ]  ! !

!QuantumNonogramArray methodsFor: 'as yet unclassified' stamp: 'hjo 2/1/2012 20:33'!
possibilities

	| indexOfUndetermined quantumToVary |
	quantumToVary := cells detect: [:each| each isDetermined not ] ifNone: [^{}].
	indexOfUndetermined := cells indexOf: quantumToVary.
	^(quantumToVary possibilities collect: [ :bool |
		[self dupCopy putBool: bool at: indexOfUndetermined] on: OutOfQuantumPossibility do: [:ex| nil].
		]) reject: [:each| each isNil ] ! !

!QuantumNonogramArray methodsFor: 'as yet unclassified' stamp: 'hjo 2/1/2012 18:54'!
printOn: aStream

	super printOn: aStream.
	cells notNil ifTrue: [
		aStream nextPut: $[.
		aStream print: cells.
		aStream nextPut: $]  
		]! !

!QuantumNonogramArray methodsFor: 'as yet unclassified' stamp: 'hjo 2/1/2012 19:47'!
putBool: aBoolean at: anInteger 

	(cells at: anInteger) be: aBoolean.
	self isDetermined 
			ifTrue: [ 
					((Array streamContents: [ :stream | 
									((cells collect: #determinedValue) as: RunArray) runsAndValuesDo: [ :r :v | v ifTrue: [ stream nextPut: r ] ] ]) = hints) ifFalse: [OutOfQuantumPossibility new signal] ]! !

Object subclass: #QuantumObject
	instanceVariableNames: 'possibilities'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QuantumLogic'!

!QuantumObject classSide methodsFor: 'instance creation' stamp: 'hjo 2/1/2012 21:33'!
fromQuantumObjects: anArray gettingBlock: aBlockClosure 

	^self unionPossibilitiesOfQuantumObjects: (anArray collect: [:qo| (aBlockClosure value: qo) ]).
	! !

!QuantumObject classSide methodsFor: 'instance creation' stamp: 'hjo 1/28/2012 17:54'!
possibilities: anArray 

	^self new possibilities: anArray  ! !

!QuantumObject classSide methodsFor: 'instance creation' stamp: 'hjo 2/1/2012 21:15'!
unionPossibilitiesOfQuantumObjects: aCollection
 
	^self possibilities: (aCollection inject: Set new into: [:set :qo| set addAll: qo possibilities; yourself]) asArray! !

!QuantumObject methodsFor: 'changing' stamp: 'hjo 2/1/2012 14:06'!
be: anObject 

	self assert: [possibilities includes: anObject].  
	possibilities := Array with: anObject! !

!QuantumObject methodsFor: 'as yet unclassified' stamp: 'hjo 2/15/2012 23:00'!
collectPossibilities: aBlockClosure 

	^Array streamContents: [:stream|
		possibilities do: [:each|  [stream nextPut: (aBlockClosure value: each)] on: OutOfQuantumPossibility do: [:exp| "ignore error"] ]
		].! !

!QuantumObject methodsFor: 'accessing' stamp: 'hjo 2/1/2012 18:24'!
determinedValue

	self assert: [self isDetermined].
	^possibilities anyOne! !

!QuantumObject methodsFor: 'testing' stamp: 'hjo 1/28/2012 17:55'!
isDetermined

	^possibilities size = 1! !

!QuantumObject methodsFor: 'accessing' stamp: 'hjo 1/28/2012 19:47'!
possibilities

	^ possibilities! !

!QuantumObject methodsFor: 'accessing' stamp: 'hjo 1/28/2012 20:42'!
possibilities: anArray 

	self assert: [anArray notEmpty].
	possibilities := anArray ! !

!QuantumObject methodsFor: 'printing' stamp: 'hjo 1/28/2012 20:25'!
printOn: aStream

	self isDetermined 
		ifTrue: [aStream print: possibilities anyOne ]
		ifFalse: [super printOn: aStream ]! !

!QuantumObject methodsFor: 'changing' stamp: 'hjo 1/28/2012 20:18'!
removePossibility: anObject

	possibilities := possibilities copyWithout: anObject.
	possibilities isEmpty ifTrue: [OutOfQuantumPossibility new signal]! !

Object subclass: #QuantumSudoku
	instanceVariableNames: 'matrix'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QuantumLogic-Example'!

!QuantumSudoku classSide methodsFor: 'as yet unclassified' stamp: 'hjo 1/28/2012 19:35'!
matrix: aMatrix

	^self new matrix: aMatrix! !

!QuantumSudoku methodsFor: 'quantumlogic' stamp: 'hjo 1/29/2012 17:31'!
isDetermined

	^matrix allSatisfy: [:each| each isDetermined ]  ! !

!QuantumSudoku methodsFor: 'accessing' stamp: 'hjo 1/28/2012 19:35'!
matrix
	^matrix! !

!QuantumSudoku methodsFor: 'accessing' stamp: 'hjo 1/28/2012 19:35'!
matrix: aMatrix

	matrix := aMatrix! !

!QuantumSudoku methodsFor: 'accessing' stamp: 'hjo 1/28/2012 19:45'!
matrixIndexOfAmbiguousElement

	self matrix withIndicesDo: [:x :r :c| 
		x isDetermined ifFalse: [^{r. c}] ].
	^nil! !

!QuantumSudoku methodsFor: 'quantumlogic' stamp: 'hjo 1/29/2012 17:30'!
possibilities

	|ambiguousIndex|
	ambiguousIndex := self matrixIndexOfAmbiguousElement.
	^ambiguousIndex isNil 
		ifTrue: [ {} ]
		ifFalse: [
			((matrix at: ambiguousIndex first at: ambiguousIndex second) possibilities collect: [:p| |cop|
				cop := self deepCopy.
				[cop putNumber: p at: ambiguousIndex first at: ambiguousIndex second] on: OutOfQuantumPossibility do: [:exp| 
					cop := nil.
				].
				cop
				]) reject: [:each| each isNil].
		]! !

!QuantumSudoku methodsFor: 'printing' stamp: 'hjo 1/28/2012 20:23'!
printOn: aStream

	super printOn:   aStream.
	aStream nextPut: $[.
	aStream print: matrix.
	aStream nextPut: $].  ! !

!QuantumSudoku methodsFor: 'changing' stamp: 'hjo 1/28/2012 20:15'!
putNumber: aNumber at: rowIndex at: columnIndex

	| determinedQuantum blockR blockC cellsAffected|
	determinedQuantum := matrix at: rowIndex at: columnIndex.
	determinedQuantum be: aNumber.
	
	cellsAffected := ((matrix atRow: rowIndex), (matrix atColumn: columnIndex), 
		[blockR := ((rowIndex - 1)//3)*3+1.
		blockC := ((columnIndex -1)//3)*3+1.
		(matrix atRows: blockR to: blockR +2 columns: blockC to:  blockC + 2) asArray] value) copyWithout: determinedQuantum.
	cellsAffected do: [:each|
		each removePossibility: aNumber  
		]! !
